SystemOrganization addCategory: #'System-Clipboard'!

!OSPlatform methodsFor: '*System-Clipboard' stamp: 'AlainPlantec 3/2/2010 14:41'!
clipboardClass
	^Clipboard! !

Object subclass: #Clipboard
	instanceVariableNames: 'contents recent'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'System-Clipboard'!

!Clipboard commentStamp: 'AlainPlantec 1/15/2010 11:42' prior: 0!
The Clipboard class is the abstract superclass for the concrete platform specific clipboard. The legacy clipboard support using the VM supplied primitives is implemented by Clipboard/ExternalClipboard.

The Clipboard implements a basic buffering scheme for text. The currently selected text is also exported to the OS so that text can be copied from and to other applications. Commonly only a single instance is used (the default clipboard) but applications are free to use other than the default clipboard if necessary.!

!Clipboard class methodsFor: 'accessing' stamp: 'ar 1/15/2001 18:45'!
chooseRecentClipping  "Clipboard chooseRecentClipping"
	"Choose by menu from among the recent clippings"
	^self default chooseRecentClipping! !

!Clipboard class methodsFor: 'accessing' stamp: 'ar 1/15/2001 18:35'!
clipboardText "Clipboard clipboardText"
	^self default clipboardText.! !

!Clipboard class methodsFor: 'accessing' stamp: 'ar 1/15/2001 18:35'!
clipboardText: aText 
	^self default clipboardText: aText! !

!Clipboard class methodsFor: 'accessing' stamp: 'michael.rueger 3/2/2009 11:12'!
default
	^Default ifNil: [Default := OSPlatform current clipboardClass new].! !

!Clipboard class methodsFor: 'initialization' stamp: 'michael.rueger 3/2/2009 11:11'!
initialize
	"Clipboard initialize"

	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self.
	self startUp: true.! !

!Clipboard class methodsFor: 'initialization' stamp: 'VeronicaUquillas 6/11/2010 13:53'!
shutDown: quitting
	"Pharo is shutting down. If this platform requires specific shutdown code, this is a great place to put it."
! !

!Clipboard class methodsFor: 'initialization' stamp: 'VeronicaUquillas 6/11/2010 13:53'!
startUp: resuming
	"Pharo is starting up. If this platform requires specific intialization, this is a great place to put it."
	resuming
		ifTrue: [Default := nil]! !

!Clipboard methodsFor: 'accessing' stamp: 'StephaneDucasse 1/29/2010 21:42'!
chooseRecentClipping  
	"Choose by menu from among the recent clippings"
	"Clipboard chooseRecentClipping"
	
	recent ifNil: [^ nil].
	^ UIManager default
		chooseFrom: (recent collect: [:txt | ((txt asString contractTo: 50)
									copyReplaceAll: Character cr asString with: '\')
									copyReplaceAll: Character tab asString with: '|'])
		values: recent! !

!Clipboard methodsFor: 'accessing' stamp: 'michael.rueger 6/10/2009 13:42'!
clipboardText
	"Return the text currently in the clipboard. If the system clipboard is empty, or if it differs from the Smalltalk clipboard text, use the Smalltalk clipboard. This is done since (a) the Mac clipboard gives up on very large chunks of text and (b) since not all platforms support the notion of a clipboard."

	| string decodedString |
	string := self primitiveClipboardText.
	(string isEmpty
			or: [string = contents asString])
		ifTrue: [^ contents].
	decodedString := string convertFromWithConverter: UTF8TextConverter new.
	decodedString := decodedString replaceAll: 10 asCharacter with: 13 asCharacter.
	^ decodedString = contents asString
		ifTrue: [contents]
		ifFalse: [decodedString asText].
! !

!Clipboard methodsFor: 'accessing' stamp: 'StephaneDucasse 12/30/2010 14:33'!
clipboardText: text 

        | string |
        string := text asString.
        self noteRecentClipping: text asText.
        contents := text asText.
        string := string convertToWithConverter: UTF8TextConverter new.
        string := string replaceAll: 13 asCharacter with: 10 asCharacter.
        self primitiveClipboardText: string.! !

!Clipboard methodsFor: 'initialize' stamp: 'alain.plantec 5/28/2009 09:47'!
initialize
	super initialize.
	contents := '' asText.
	recent := OrderedCollection new! !

!Clipboard methodsFor: 'private' stamp: 'ar 1/15/2001 18:34'!
noteRecentClipping: text
	"Keep most recent clippings in a queue for pasteRecent (paste... command)"
	text isEmpty ifTrue: [^ self].
	text size > 50000 ifTrue: [^ self].
	(recent includes: text) ifTrue: [^ self].
	recent addFirst: text.
	[recent size > 5] whileTrue: [recent removeLast].
! !

!Clipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/25/2009 14:23'!
primitiveClipboardText
	"Get the current clipboard text. Return the empty string if the primitive fails."
	<primitive: 141>
	^ ''! !

!Clipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/25/2009 14:23'!
primitiveClipboardText: aString
	"Set the current clipboard text to the given string."

	<primitive: 141>
	"don't fail if the primitive is not implemented"! !

Clipboard subclass: #ExternalClipboard
	instanceVariableNames: 'clipboard'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Clipboard'!

!ExternalClipboard commentStamp: 'michael.rueger 3/2/2009 13:25' prior: 0!
An ExternalClipboard is the abstract superclass for the platform specific clipboards based on the clipboard plugin (former ExtendedClipboardInterface originally developed for Sophie).

Instance Variables
	clipboard:		SmallInteger

clipboard
	- handle for the external clipboard. If 0 the external clipboard is invalid
!

!ExternalClipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/25/2009 14:22'!
addClipboardData: data dataFormat: aFormat
	clipboard = 0 ifTrue: 
		[Clipboard clipboardText: data asString.	
		^self].
	self primAddClipboardData: clipboard data: data dataFormat: aFormat! !

!ExternalClipboard methodsFor: 'accessing' stamp: 'michael.rueger 3/2/2009 13:42'!
clearClipboard
	clipboard = 0 ifTrue: [^self].
	^ self primClearClipboard: clipboard.! !

!ExternalClipboard methodsFor: 'accessing' stamp: 'michael.rueger 6/10/2009 13:42'!
clipboardText
	"Return the text currently in the clipboard. If the system clipboard is empty, or if it differs from the Smalltalk clipboard text, use the Smalltalk clipboard. This is done since (a) the Mac clipboard gives up on very large chunks of text and (b) since not all platforms support the notion of a clipboard."

	| decodedString bytes |
	clipboard = 0 ifTrue: 
		[^super clipboardText].
	bytes := self primReadClipboardData: clipboard format: 'public.utf8-plain-text'.
	bytes
		ifNil: [^super clipboardText].
	decodedString := bytes asString convertFromWithConverter: UTF8TextConverter new.
	decodedString := decodedString replaceAll: 10 asCharacter with: 13 asCharacter.
	^decodedString = contents asString
		ifTrue: [contents]
		ifFalse: [decodedString asText].
! !

!ExternalClipboard methodsFor: 'accessing' stamp: 'michael.rueger 3/25/2009 14:47'!
clipboardText: text 

	| string data |
	string := text asString.
	self noteRecentClipping: text asText.
	contents := text asText.
	data := (string convertToWithConverter: UTF8TextConverter new) asByteArray.
	clipboard = 0 ifTrue: 
		[^super clipboardText: text].
	self clearClipboard.
	self primAddClipboardData: clipboard data: data dataFormat: 'public.utf8-plain-text'! !

!ExternalClipboard methodsFor: 'private' stamp: 'michael.rueger 3/2/2009 13:42'!
createClipboard
	clipboard = 0 ifTrue: [^self].
	^ self primCreateClipboard.! !

!ExternalClipboard methodsFor: 'initialize' stamp: 'StephaneDucasse 8/30/2009 14:55'!
initialize
	super initialize.
	clipboard := [self createClipboard] on: Error do: [:ex | clipboard := 0]! !

!ExternalClipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/25/2009 14:25'!
primAddClipboardData: aClipboard data: data dataFormat: aFormat

	<primitive:'ioAddClipboardData' module: 'ClipboardExtendedPlugin'>
	^ self primitiveFailed! !

!ExternalClipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/2/2009 13:42'!
primClearClipboard:  aClipboard
	<primitive:'ioClearClipboard' module: 'ClipboardExtendedPlugin'>
	^ self primitiveFailed.
! !

!ExternalClipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/2/2009 13:42'!
primCreateClipboard
	<primitive:'ioCreateClipboard' module: 'ClipboardExtendedPlugin'>
	^ self primitiveFailed.
! !

!ExternalClipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/25/2009 14:25'!
primGetClipboardFormat: aClipboard formatNumber: formatNumber

	<primitive:'ioGetClipboardFormat' module: 'ClipboardExtendedPlugin'>
	^ self primitiveFailed! !

!ExternalClipboard methodsFor: 'primitives' stamp: 'marcus.denker 6/11/2009 12:24'!
primReadClipboardData: aClipboard format: format

	<primitive:'ioReadClipboardData' module: 'ClipboardExtendedPlugin'>
	^ self primitiveFailed! !

ExternalClipboard subclass: #MacOSClipboard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Clipboard'!

!MacOSClipboard methodsFor: 'private' stamp: 'michael.rueger 3/2/2009 12:51'!
addUF8StringClipboardData: aString
	| ba  |

	self clearClipboard.
	ba := aString convertToWithConverter: (UTF8TextConverter new).
	self addClipboardData: ba dataFormat: 'public.utf8-plain-text'
! !

!MacOSXPlatform methodsFor: '*System-Clipboard' stamp: 'michael.rueger 3/2/2009 10:56'!
clipboardClass
	^MacOSClipboard! !
Clipboard initialize!
